import uuid

from flask_login import current_user
from flask import jsonify
from sqlalchemy.orm import sessionmaker
from app import db
from datetime import datetime


from components.k2site.k2site.models import K2Approval, K2users
from k2.k2obj import K2Obj
from .k2mail import K2Mail

class K2Notifications(K2Obj):
    name = 'k2notifications'

    def __init__(self, *args, **kwargs):
        super().__init__()

    @classmethod
    def __generate_id(cls):
        '''Генерація ID'''
        generated_id = uuid.uuid4().hex[:32]
        return ''.join([generated_id[i:i + 4] for i in range(0, len(generated_id), 4)])

    @classmethod
    def save_approve_obj(cls, obj_id, user_list, url):
        if not obj_id or not url or not user_list:
            return jsonify({"error": "Object ID, URL or user list is missing"}), 400
        Session = sessionmaker(bind=cls.db)
        session = Session()
        try:
            # Проходимо по кожному користувачеві і створюємо запис у таблиці K2Approval
            for user_id in user_list:
                approval_id = cls.__generate_id()  # Генеруємо унікальний approval_id
                # Створюємо новий запис про погодження в таблиці
                new_approval = K2Approval(
                    approval_id=approval_id,
                    object_id=obj_id,
                    user_id=user_id,
                    is_approved=False,  # Спочатку статус не затверджено
                    description=f"Approval request for object {obj_id}",
                    approval_url=url,
                    createuser=current_user.get_id(),
                    createdate=datetime.utcnow(),
                    active=1
                )
                # Зберігаємо у базі даних
                session.add(new_approval)
                try:
                    session.commit()
                    print('Commit successful')
                except Exception as commit_error:
                    print(f'Commit failed: {commit_error}')
                    session.rollback()
            return jsonify({"status": "success", "message": "Approval requests created"}), 200
        except Exception as e:
            return jsonify({"error": str(e)}), 500
        finally:
            session.close()


    @classmethod
    def send_approval_request(cls, obj_id, user_list, url, description):
        """
        Функція для відправки запиту на затвердження документу
        Відправляє запит від імені користувача
        """
        try:
            # Спочатку зберігаємо запити на затвердження
            cls.save_approve_obj(obj_id, user_list, url)

            # Відправляємо email
            subject = 'Затвердження документа'
            email_template = 'Вам потрібно затвердити документ за посиланням.'

            # Отримуємо користувачів зі списку для погодження
            users = K2users.query.filter(K2users.user_id.in_(user_list), K2users.active == 1).all()
            for user in users:
                K2Mail().send_email(subject, user.email, email_template, K2.domain)

            return jsonify({"status": "success", "message": "Approval requests and emails sent"}), 200

        except Exception as e:
            db.session.rollback()  # Відкочуємо транзакцію в разі помилки
            return jsonify({"error": str(e)}), 500
